function out = mapcurves(A,B,maplbl)
%
% matlab implementation of the mapcurves goodness of fit measure 
% (Hargrove et al. 20006, see full reference below) for comparing 
% two categorical maps
%
% usage: 
%
%   out = mapcurves(mapA,mapB,[maplbl])
%
% where 
%   mapA = a matrix or vector with integers, representing a categorical map
%   mapB = a matrix or vector with integers, representing a categorical map
%   maplbl = a cell array with names of the two maps (default: 'A' and 'B')
%   
%   out = a structural array with the outputs 
%       out.GOF = the mapcurves goodness of fit value (GOF)
%       out.Refmap = the map to be used as reference map ('A' or 'B')
%       out.GOFtable = the GOF for each pair of classes in maps A and B
%                 the rows match with the classes in map A, the columns 
%                 with the classes in map B
%       out.lblA = vector with class labels for map A (matching with rows in GOFtable) 
%       out.lblB = vector class labels for map B (matching with colums in GOFtable)
%       out.BMC_A2B = 3-column matrix with best matching class (BMC) and 
%                       corresponding GOF, when using A as reference 
%       out.BMC_B2A = 3 column matrix BMC and corresponding GOF when using 
%                       B as reference
%       out.lblBMC = cell array with labels for the three columns in
%                        the fields BMC_A2B and BMC_B2A
%
%  additional requirements:
%    - maps A and B should be matrices or vectors of equal size 
%    - missing values should be coded as NaN and are disregarded 
%       in the comparison
%
% Calling mapcurves withoutinput arguments, will run a small demo.
%
%
% % example 1
%
% A = round(5*rand(15));
% B = round(8*rand(15));
% mapcurves(A,B)
%
% % example 2
% 
% mapcurves    % a demonstration of 4 cases, 
%              % still to be done: more explanation & figures
%
% 
% This implementation is based on the description of the Mapcurves algorithm
% by Hargrove et al. in the following paper:
%
% William W. Hargrove, Forrest M. Hoffman and Paul F. Hessburg (2006) 
%   Mapcurves: a quantitative method for comparing categorical maps.
%   J Geograph Syst, 8, 187208. DOI 10.1007/s10109-006-0025-x

% Emiel van Loon, May 2011
% University of Amsterdam
% http://staf.science.uva.nl/~vanloon/

if nargin<2
    mapcurves_demo
    return
end

if nargin<3
    maplbl = {'A','B'};
end

a = unique(A(:));
b = unique(B(:));

% to remove NaN-values:
a = a(isfinite(a));
b = b(isfinite(b));

nra = length(a);
nrb = length(b);
C = nan(nra,nrb);
rC = nan(nra,nrb);

for i=1:nra;
    for j=1:nrb;
        C(i,j) = sum( ( A(:)==a(i) ) & ( B(:)==b(j) ) );
    end
end

Sa = sum(C,2)';
Sb = sum(C,1);

for i=1:nra;
    for j=1:nrb;
        rC(i,j) = ( C(i,j)^2 ) / ( Sa(i)*Sb(j) );
    end
end

rSa = sum(rC,2);
rSb = sum(rC,1);

sSa = [0 sort(rSa)' 1];
sSb = [0 sort(rSb) 1];

PCLa = [1 linspace(1,0,nra) 0]; % percent of categories larger than this GOF value
PCLb = [1 linspace(1,0,nrb) 0];

figure, plot(sSa, PCLa, 'b', sSb, PCLb, 'r')
legend( [maplbl{1} ' as reference'], [maplbl{2} ' as reference'] )
xlabel('GOF score')
ylabel('% of map classes < GOF score')

GOFa = max( trapz(sSa,PCLa) );
GOFb = max( trapz(sSb,PCLb) );

% % Identical to trapz, for GOFb:
%
% GOFb_x=0;
% for i=1:nra
%  GOFb_x = GOFb_x + ( sSb(i+1) - sSb(i) ) * PCLb(i+1)  + ...
%               0.5 * ( sSb(i+1) - sSb(i) ) * ( PCLb(i) - PCLb(i+1) );
% end			  
% GOFb_x

[GOF,idx] = max( [GOFa, GOFb] );

if idx==1, ref = maplbl{1}; end
if idx==2, ref = maplbl{2}; end

BMC_A2B = nan(nra,3);
BMC_B2A = nan(nrb,3);

BMC_A2B(:,1) = a;
for i = 1:nra
  [BMC_A2B(i,3), BMC_A2B(i,2)] = max( rC(i,:) );
end

BMC_B2A(:,2) = b;
for j = 1:nrb
  [BMC_B2A(j,3), BMC_B2A(j,1)] = max( rC(:,j) );
end

out.GOF = GOF;
out.Refmap = ref;
out.GOFtable = rC;
out.lblA = a;
out.lblB = b';
out.BMC_A2B = BMC_A2B;
out.BMC_B2A = BMC_B2A;
out.lblBMC = {['class in ' maplbl{1}], ['class in ' maplbl{2}], 'GOF for class-pair'};


%%

function mapcurves_demo
%
% this demo is incomplete - more explanation and informative graphs are
% being needed for each example  -- still to be done.
% 

% A1 and A2 are random, with the samen number of classes
% B1 and B2 are equivalent, up to the point that B2 has one extra class 
%   which is fully contained in a B1-class
% B2 and B3 have an equal number of classes, there are slight shifts at the 
%   class boundaries

A1 = [4    4    6    0    7    6    1    0    2     NaN
      4    0    8    0    8    5    0    3    3     1
      4    1    6    8    1    6    6    1    8     2
      5    2    1    7    0    2    7    1    8     2
      3    3    5    2    8    0    6    8    6     5
      4    5    5    1    7    0    0    1    7     6
      6    6    7    7    0    2    0    2    3     0
      8    7    2    4    7    5    6    1    6     0
      0    1    3    8    8    2    8    1    7     8
      4    8    7    6    5    0    7    6    2     3];
	
A2 =[NaN  6    3    0    6    3    3    1    6     8
     7   NaN   4    0    3    7    6    0    8     7
     3    3    1    4    3    4    3    6    0     5
     0    4    4    6    7    0    3    3    2     7
     7    4    4    7    7    7    2    8    8     7
     8    2    1    1    8    8    6    6    7     1
     6    8    1    4    1    7    4    7    6     5
     0    5    0    2    1    1    2    7    5     7
     7    0    4    1    6    0    5    2    0     6
     4    6    4    7    5    3    8    0    5     8];

B1 = [ 1    1    1    1    1    1    1    1    1     1
      1    1    1    1    1    1    2    2    2     1
      1    1    1    1    1    1    2    2    2     1
      1    1    1    1    1    1    2    2    2     1
      3    3    1    1    1    2    2    2    2     1
      3    3    1    1    1    2    2    2    1     1
      3    3    3    1    1    1    1    1    1     1
      3    3    3    1    1    1    1    1    1     1
      3    3    3    1    1    1    1    1    4     1
      1    1    1    1    1    1    1    1    4     4 ]; 
	  
B2 = [ 1    1    1    1    1    1    1    1    1     1
      1    1    1    1    1    1    2    5    5     1
      1    1    1    1    1    1    2    5    5     1
      1    1    1    1    1    1    2    5    5     1
      3    3    1    1    1    2    2    5    5     1
      3    3    1    1    1    2    2    5    1     1
      3    3    3    1    1    1    1    1    1     1
      3    3    3    1    1    1    1    1    1     1
      3    3    3    1    1    1    1    1    4     1
      1    1    1    1    1    1    1    1    4     4 ];	  
	  
B3 = [ 1    1    1    1    1    1    1    2    1     1
      1    1    1    1    1    1    2    2    5     1
      1    1    1    1    1    1    2    5    5     1
      1    1    1    1    1    2    2    5    5     1
      3    3    1    1    1    2    2    5    5     1
      3    3    1    1    1    2    2    5    1     1
      3    3    3    1    1    1    1    1    1     1
      3    3    3    1    1    1    1    1    1     1
      3    3    3    1    1    1    1    1    4     1
      3    3    3    1    1    1    1    4    4     4 ]; 	  


disp(' ')
disp('Four simple map-comparison examples are being shown')
disp(' ')
disp(' ---- press a button to continue ----')
disp(' ')
pause

disp(' ')
disp('Compare two random matrices with integer values from 0 to 8')
disp('There are also a few NaN-entries in both matrices')
disp(' ')
disp('>> C_rnd = mapcurves(A1,A2) ')
disp(' ')
C_rnd = mapcurves(A1,A2)
disp(' ---- press a button to continue ----')
disp(' ')
pause
close(gcf)

disp(' ')
disp('Compare two maps with are equivalent, up to the point that')
disp('the second map has one extra class which is fully contained')
disp('in one class of the first map')
disp(' ')
disp('>> C_eq = mapcurves(B1,B2) ')
disp(' ')
C_eq = mapcurves(B1,B2)
disp(' ---- press a button to continue ----')
disp(' ')
pause
close(gcf)

disp(' ')
disp('Compare two maps which match well (but not perfect) and') 
disp('have an equal number of classes.')
disp(' ')
disp('>> C_sh1 = mapcurves(B2,B3) ')
disp(' ')
C_sh1 = mapcurves(B2,B3)
disp(' ---- press a button to continue ----')
disp(' ')
pause
close(gcf)

disp(' ')
disp('Compare two maps which match well (but not perfect) and') 
disp('have an unequal number of classes.')
disp(' ')
disp('>> C_sh2 = mapcurves(B1,B3) ')
disp(' ')
C_sh2 = mapcurves(B1,B3)
disp(' ---- press a button to finish this demo ----')
disp(' ')
pause
close(gcf)
